#ifndef _LISTA_H_
#define _LISTA_H_
#include "stdafx.h"

#include "Elemento.h"

template<class TIPO>
class Lista
{
private:
	Elemento<TIPO>*  pPrimeiro;
	Elemento<TIPO>*  pAtual;
	
	int qtdElementos;
public:
	Lista();
	~Lista();
	bool incluir( TIPO pInfo );
	Elemento<TIPO>* getPrimeiro();
	Elemento<TIPO>* getUltimo();

	bool gravarEmArquivo(char const * const arquivo);
	bool recuperarDeArquivo(char const * const arquivo);
};

template<class TIPO>
Lista<TIPO>::Lista()
{
	pPrimeiro = NULL;
	pAtual    = NULL;
	qtdElementos = 0;
}

template<class TIPO>
Lista<TIPO>::~Lista()
{
	Elemento<TIPO>* aux = pPrimeiro;
	while (aux != NULL)
	{
		pPrimeiro = aux->getProximo();
		delete aux;
		aux = pPrimeiro;
	}
}

template<class TIPO>
bool Lista<TIPO>::incluir( TIPO pInfo )
{
	Elemento<TIPO>* pElemento = new Elemento<TIPO>();
	pElemento->setInfo( pInfo );
	pElemento->setProximo( NULL );
	pElemento->setAnterior( NULL );
	
	if (NULL == pPrimeiro)
	{
		pAtual = pPrimeiro = pElemento;		
	}
	else
	{
		pAtual->setProximo( pElemento );
		pElemento->setAnterior( pAtual );
		pAtual = pElemento;
	}

	qtdElementos ++;		
	return true;
}

template<class TIPO>
Elemento<TIPO>* Lista<TIPO>::getPrimeiro()
{
	return pPrimeiro;
}

template<class TIPO>
Elemento<TIPO>* Lista<TIPO>::getUltimo()
{
	return pAtual;
}

template<class TIPO>
bool Lista<TIPO>::gravarEmArquivo(char const * const arquivo)
{
	bool gravavel = true;
	if (NULL == arquivo || 0 == strcmp(arquivo, ""))
		gravavel = false;
	
	if (gravavel)
	{
		// recupera os elementos da lista;
		TIPO* buffer = new TIPO[qtdElementos];
		Elemento<TIPO>* aux = pPrimeiro;
		for (int i = 0; i < qtdElementos; i++)
		{
			buffer[ i ] = aux->getInfo();
			aux = aux->getProximo();
		}

		// tenta abrir o arquivo;
		ofstream saida(arquivo, ios::binary | ios::out);
		if (!saida)
			gravavel = false;
		
		if (gravavel)
		{	// grava;
			saida.write(reinterpret_cast< char* >(buffer), qtdElementos * sizeof(TIPO));
		}
		saida.close();
		
		delete[] buffer;
	}

	return gravavel;
}

template<class TIPO>
bool Lista<TIPO>::recuperarDeArquivo(char const * const arquivo)
{
	if (NULL == arquivo || 0 == strcmp(arquivo, ""))
		return false;
	
	// tenta abrir o arquivo;
	ifstream entrada(arquivo, ios::binary | ios::in);
	if (!entrada)
		return false;

	TIPO buffer;
	// l um por um dos elementos e insere na lista;
	while( !entrada.eof() )
	{
		entrada.read(reinterpret_cast< char* >(&buffer), sizeof(TIPO));
		this->incluir( buffer );
	}
	entrada.close();

	return true;
}

#endif
